% R6P MEX EXAMPLE
% calculates absolute pose for 2 images with various rolling shutter effects
% 3D points and 2D image points are provided for the computation
% images are provided for visual verification
% 
% comparison of P3P and R6P
%
% a simple version of RANSAC is used for robust estimation
%
% this code demonstrates the use of R6P algorithm for Rolling shutter cameras
%
% please refere to the paper
% 
% Cenek Albl, Zuzana Kukelova, Tomas Pajdla:
% R6P - Rolling Shutter Absolute Camera Pose, 
% The IEEE Conference on Computer Vision and Pattern Recognition (CVPR),
% June, 2015, Boston, USA.
%
% 2015, Cenek Albl, alblcene@cmp.felk.cvut.cz

addpath('utils')
%calibration matrix 
K = [ 1485.2 0 639.5;
      0 1485.2 359.5;
      0 0 1];
% RANSAC settings
ransac_rounds = 100;
ransac_thr = 0.002;
%load data
load('data/seq20_data');
img{1} = imread('data/Image125.jpg'); 
img{2} = imread('data/Image170.jpg'); 

scores =[];
for i =1:size(rs_data,2)
    %run P3P RANSAC to initialize
    X = rs_data{i}.X;
    u = rs_data{i}.u;
    % transform to the camera euclidean system            
    u = h2a(K\a2h(u));
    [P3P_model,P3P_set,P3P_score] = ransac_P3P(X,u,ransac_rounds,ransac_thr);
    %P3P recovered camera
    P_P3P{i} = K*[P3P_model.R P3P_model.t];
    % run R6P RANSAC 
    %rotate the 3D points using the camera rotation matrix found by P3P
    %P3P
    X = P3P_model.R*X;
    % R6P is built for a HORIZONTAL rolling shutter direction
    % i.e. lines are read from left to right (along x axis)
    % in order to use it for images where RS was in VERTICAL direction
    % simply transform the scene to interchange x and y axes
    RSW = [0 1 0; 1 0 0; 0 0 1];
    X = RSW*X;
    u=u([2 1],:);
    [R6P_model,R6P_set,R6P_score] = ransac_R6P(X,u,0,ransac_rounds,ransac_thr);
    % return the result back to correct coord.sys. 
    R = R6P_model.R*RSW*P3P_model.R;
    P_R6P{i} = K*RSW*[R R6P_model.C];
    scores(:,i) = [P3P_score; R6P_score];
    figure
    subplot(1,2,1);
    imshow(img{i});
    hold on
    plot(rs_data{i}.u(1,:),rs_data{i}.u(2,:),'.k','MarkerSize',15,'LineWidth',10);
    plot(rs_data{i}.u(1,P3P_set),rs_data{i}.u(2,P3P_set),'.r','MarkerSize',15);
    title('P3P inliers')
    subplot(1,2,2);
    imshow(img{i});
    hold on
    plot(rs_data{i}.u(1,:),rs_data{i}.u(2,:),'.k','MarkerSize',15,'LineWidth',10);
    plot(rs_data{i}.u(1,R6P_set),rs_data{i}.u(2,R6P_set),'.g','MarkerSize',15);
    title('R6P inliers')
    pause(0.1)
end
%plot results
figure
plot3(rs_data{1}.X(1,:),rs_data{1}.X(2,:),rs_data{1}.X(3,:),'.b');
hold on
plot3(rs_data{2}.X(1,:),rs_data{2}.X(2,:),rs_data{2}.X(3,:),'.b');
axis equal
axis([-20 20 -30 30 -30 10])
draw_camera(P_P3P{1},'r',2);
draw_camera(P_P3P{2},'r',2);
draw_camera(P_R6P{1},'g',2);
draw_camera(P_R6P{2},'g',2);
title('3D points and recovered cameras')