% A simple RANSAC routine for finding absolute pose of a calibrated
% perspective camera (P3P)
%
% 2015, Cenek Albl, alblcene@cmp.felk.cvut.cz
%
% [best_model,score] = ransac_P3P(X,u,iters,thr)
%
% INPUT
%
% iters - number of RANSAC iterations
% thr - RANSAC threshold
% X - 3D points
% u - 2D correspondences
%
% OUTPUT
%
% best_model - R and t of the camera with the highest score
% score - the number of points in the consensus set

function [best_model, best_set,score] = ransac_P3P(X,u,iters,thr)
    n=size(u,2);
    minpts = 3;
    iter = 0;
    score = 0;
    best_set=[];
    best_model=[];
    while(iter<iters)
        %select 3 distinctive pts randomly
        idxs = randperm(n);
        idxs = idxs(1:minpts);
        %compute P3P
        models = p3p_grunert(X(:,idxs),u(:,idxs));
        for m = models
            Xc = m{1};
            [R t] = XX2Rt_simple( X(:,idxs), Xc );
            consensus_set = [];
            for j=1:size(u,2)
                %reprojection error 
                proj = h2a(R*X(:,j)+t);
                err = norm(proj-u(:,j));
                %check if the point fits with error smaller than thr
                if(err<thr)
                    consensus_set = [consensus_set j];
                end
            end
            %check if more points in consensus set
            if(size(consensus_set,2)>score)
                %we found new best candidate
                score = size(consensus_set,2);
                best_model.R = R;
                best_model.t = t;
                best_set = consensus_set;
            end
        end
        iter = iter+1;
    end
end