% A simple RANSAC routine for finding the absolute pose for a rolling
% shutter camera using R6P
%
% Cenek Albl, Zuzana Kukelova, Tomas Pajdla:
% R6P - Rolling Shutter Absolute Camera Pose, 
% The IEEE Conference on Computer Vision and Pattern Recognition (CVPR),
% June, 2015, Boston, USA.
%
% 2015, Cenek Albl, alblcene@cmp.felk.cvut.cz
%
% [best_model,score] = ransac_R6P(X,u,iters,thr)
%
% INPUT
%
% iters - number of RANSAC iterations
% thr - RANSAC threshold
% X - 3D points
% u - 2D correspondences
% r0 - the image coordinate around which the RS model is linearized (see the paper),
% default: 0 (center of the image) 
%
% OUTPUT
%
% best_model - R and t of the camera with the highest score
% score - the number of points in the consensus set


function [best_model,best_set,score] = ransac_R6P(X,u,r0,iters,thr)
    % k - num_iterations
    % X - 3D points
    % u - 2D correspondences
    n=size(u,2);
    minpts = 6;
    iter = 0;
    score =0;
    best_set=[];
    best_model=[];
    while(iter<iters)
        %select 6 pts randomly
        idxs = randperm(n);
        idxs = idxs(1:minpts);
        [vr,wr,tr,Cr,R] = r6p_mex(X(:,idxs),u(:,idxs),r0);
        %tic
        for i=1:size(vr,2)
            v = vr(:,i);
            t = tr(:,i);
            C = Cr(:,i);
            w = wr(:,i);
            consensus_set = [];
            % eliminate solutions with unreasonably high camera movement
            if(norm(w)>1)
                continue;
            end
            for j=1:size(u,2)
                r = u(1,j);
                Rw = (eye(3)+X_((r-r0)*w));
                Rv = (eye(3)+X_(v));
                Rres = Rw*Rv;
                Cres = C+(r-r0)*t;
                proj = Rres*X(:,j)+Cres;
                proj = [proj(1)/proj(3); proj(2)/proj(3)];
                err = norm(proj-u(:,j));

                %check if the point fits with error smaller than thr
                if(err<thr)
                    consensus_set = [consensus_set j];
                end
            end
            %check if more points in consensus set
            if(size(consensus_set,2)>score)
                %we found new best candidate
                score = size(consensus_set,2);
                best_model.R = Rres;
                best_model.C = Cres;
                best_model.t = t;
                best_model.w = w;     
                best_set = consensus_set;
            end
        end
        iter= iter+1;
    end
end